/*
 * ***************************************************************************
 * MC = < Manifold Code >
 * Copyright (C) 1994--2002  Michael Holst
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * rcsid="$Id: mypde.c,v 1.1 2003/03/08 00:13:48 fetk Exp $"
 * ***************************************************************************
 */

/*
 * ***************************************************************************
 * File:     mypde.c
 *
 * Purpose:  Class PDE: methods.
 *
 *     Problem type:          time-dependent, Diffusion-reaction
 *     Spatial dimension:     2 or 3
 *     Product dimension:     1
 *     Differential operator: Linear or nonlinear potential equation
 *                            with symmetric or nonsymmetric linear part
 *     Boundary conditions:   zero or nonzero Dirichlet conditions
 *                            nonzero or zero Neumann/Robin conditions
 *
 *     Note: Equation for component i is defined as
 *
 *           sum(del(A_ij del(u_j))) = B_i   in omega
 *         -n dot sum(A_ij del(u_j)) = C_i   along natural boundaries
 *
 *           _____
 *           \      ____       ____
 *            \     \  /. A_ij \  / u_j = B_i   in omega
 *            /      \/         \/
 *           /____
 *             j
 *
 *           ____
 *           \               ____
 *            \    -n . A_ij \  / u_j = C_i     along natural boundaries
 *            /               \/
 *           /___
 *             j
 *
 * Author:   Jason Suen, modified after Michael Holst, Nathan Baker and Kaihsu Tai
 * ***************************************************************************
 */

#include "smol.h"

VEMBED(rcsid="$Id: mypde.c,v 1.1 2003/03/08 00:13:48 fetk Exp $")

VPRIVATE double flux[3], ndotflux, Diff, expp, paramM, paramA, tcurv;
VPRIVATE double U[MAXV], dU[MAXV][3], dPot[3], ddU[3], nvec[3], xq[3], curv[3];
VPRIVATE int fType, ekey;
VPRIVATE TT *myt;
VPRIVATE Vmodel *model;

VPRIVATE void initAssemble(PDE *thee, int ip[], double rp[]) {
    model = usr_getModel(thee);
    Diff = model->diff;
    paramM= rp[0];
    paramA= rp[1];
}

VPRIVATE void initElement(PDE *thee, int elementType,
        int chart, double tvx[][3], void *data){
        fType=0; 
        myt = (TT *)data;
}

VPRIVATE void initFace(PDE *thee, int faceType, int chart, double tnvec[]){

    int i, face;

    face = myt->focusFace;
    for (i=0; i<thee->dim; i++) nvec[i] = myt->nvec[face][i];
    fType = faceType;
}

VPRIVATE void initPoint(PDE *thee, int pointType, int chart, double txq[],
           double tU[], double tdU[][3]) {

    int i, j;
    double Pot;

    /* Load up the PMF gradient values here */
    for (i=0; i<thee->dim; i++) {
        Pot = 0.;
        dPot[i] = 0.;
        ddU[i] = 0.;
        dU[0][i]  = tdU[0][i];
        xq[i]     = txq[i];
    }

    for (j=0; j<myt->dimV; j++) {
         Pot += myt->phi[j]*(model->PMF)[myt->vid[j]];
         for (i=0; i<thee->dim; i++) {
             ddU[i] += myt->phix[j][i]*dU[0][i];
         }
    }


    U[0] = tU[0]; ndotflux=0;

    expp=VEXP(-Pot);

    /* boundary form case */

    for (i=0; i<thee->dim; i++) {
        flux[i]   = -Diff*expp*dU[0][i];
        ndotflux += nvec[i]*flux[i];
    }
}

VPRIVATE void Fu(PDE *thee, int key, double F[]) { 
    int i;

    switch(key) {
    case 0:
        F[0] = 0.;
        for (i=0; i<thee->dim; i++) {
            F[0] += ddU[i] - dU[0][i]*dPot[i];
        }
        F[0] *= Diff*expp;
        break;
    case 1:
        if (fType == IB_NR) {
           for (i=0; i<thee->dim; i++) {
               F[0] += dU[0][i]*nvec[i];
           }
           F[0] *= Diff*expp;
        } else if (VNEUMANN(fType)) { F[0] = 0.0; }
        else {
           F[0] = 0.0;
        }
        break;
    case 2:
        F[0] = 0.0;
        for (i=0; i<thee->dim; i++) {
            F[0] += dU[0][i]*nvec[i];
        }        
        F[0] *= Diff*expp/myt->dimV;
        break;
    default:
        F[0] = 0.0;
        break; 
    }
}

VPRIVATE double Ju(PDE *thee, int key) { 
    int i;
    double value=0;

    switch(key) {
    case 0:                 /* Reactive flux integral */
        if (fType == IB_RD) {
            value=ndotflux;
//            Vnm_print(0,"ndotflux = %le\n",ndotflux);
        }
        break;
    case 1:
        if (fType == IB_RD) value = 1;
        break;
    case 2:
        if (fType == 0) value = U[0];
        break;
    case 3:
        if (fType == 0) value = 1.0;
        break;
    case 4:        /* Flux evaluated throught volume integral */
        for (i=0; i<thee->dim; i++) value -= dU[0][i]*flux[i];
        break;
    default:
        value = 0.0;
        break;
    }

    return value;
}

VPRIVATE double Fu_v(PDE *thee, int key, double V[], double dV[][3]) {

    double value = 0.;
    int i;

    if (key==0)  for (i=0; i<thee->dim; i++) value -= dV[0][i]*flux[i];
    else if (key == 1) {
            value = 0.0;
    }
    return value;
}

VPRIVATE double DFu_wv(PDE *thee, int key, double W[], double dW[][3], 
        double V[], double dV[][3]) {

    int i;
    double value = 0., value2 = 0.;

    if (key == 0){
        for (i=0; i<thee->dim; i++) {
            value2 += dV[0][i]*dW[0][i];
        }
        if (model->tmkey == 2) {
            value = expp*(paramM*V[0]*W[0] + paramA*Diff*value2);
        } else {
            value = Diff*expp*value2;
        }
    }else if (key == 1) {
        value = 0.0;
    }else {
        Vnm_print(2, "DFu_wv: ERROR!  Don't know how to handle \
key = %d (do you really want the dual problem?)\n", key);
        VASSERT(0);
    }
    return value;
}

VPRIVATE double p_wv(PDE *thee, int key, double W[], double V[]) {

    double value = 0.;

    /* PRIMAL: interior form case */
    if (key == 0) {
        value = W[0] * V[0];
    /* PRIMAL: boundary form case */
    } else if (key == 1) {
        value = 0.0;
    /* DUAL: interior form case */
    } else if (key == 2) {
        value = 0.0;
    /* DUAL: boundary form case */
    } else if (key == 3) {
        value = 0.0;
    } else { VASSERT(0); }

    return value;
}

VPRIVATE void u_D(PDE *thee, int type, int chart, double txq[], double F[]) { 

    Vmodel *model = usr_getModel(thee);
    double value, conc = model->conc;
    double charge = model->charge;
    Vmgrid *mgrid = model->mgrid;
    
    F[0] = 0.;

    if (VDIRICHLET(type)){          /* Incorporate Dirichlet BC */
        if (type == 1)  F[0] = 0.0;
        if (type == 5)  {
/*
           if (!Vmgrid_eval(mgrid, txq, &Pot, dPot, curv)){
               Vnm_print(2, " Vmgrid_gradient failed!\n");
               VASSERT(0);
           }
*/
           F[0] = conc;
        }
    } else {                        /* Incorporate Initial Condition */
          F[0] = 0.0;
//      } 
      /*if (!Vmgrid_value(mgrid, txq, &value)){
        Vnm_print(2, "Vmgrid_value failed!\n");
        VASSERT(0);
      }
      F[0] = exp(-charge*value); */
    } 
}

VPRIVATE void u_T(PDE *thee, int type, int chart, double txq[], double F[], double dF[][3])
{ F[0] = 0.0; }

VPRIVATE void delta(PDE *thee, int type, int chart, double txq[], void *user, double F[])
{ F[0] = 0.0; }

VPRIVATE void bisectEdge(int dim, int dimII, int edgeType, int chart[], 
        double vx[][3]) {
    int i;
    for (i=0; i<dimII; i++) vx[2][i] = .5 * (vx[0][i] + vx[1][i]);
}

VPRIVATE int markSimplex(int dim, int dimII, int simplexType, int faceType[4], 
  int vertexType[4], int chart[], double vx[][3], void *data) 
{ 
  return 0;
}
VPRIVATE void oneChart(int dim, int dimII, int objType, int chart[],
        double vx[][3], int dimV) {}
VPRIVATE void mapBoundary(int dim, int dimII, int vertexType, int chart, double vx[3]){}

VPUBLIC PDE* myPDE_ctor() {

    PDE *thee = VNULL;
    int i;

    /* Create some space for the pde object */
    thee = Vmem_malloc(VNULL, 1, sizeof(PDE) );
    /* PDE-specific parameters and function pointers */
    thee->initAssemble = initAssemble;  /* once-per-assembly initialization */
    thee->initElement  = initElement;   /* once-per-element initialization  */
    thee->initFace     = initFace;      /* once-per-face initialization     */
    thee->initPoint    = initPoint;     /* once-per-point initialization    */
    thee->Fu           = Fu;            /* nonlinear strong form            */
    thee->Ju           = Ju;            /* nonlinear energy functional      */
    thee->Fu_v         = Fu_v;          /* nonlinear weak form              */
    thee->DFu_wv       = DFu_wv;        /* bilinear linearization weak form */
    thee->p_wv         = p_wv;          /* bilinear mass density form       */
    thee->delta        = delta;         /* delta function source term       */
    thee->u_D          = u_D;           /* dirichlet func and initial guess */
    thee->u_T          = u_T;           /* analytical soln for testing      */
    thee->vec          = 1;             /* unknowns per spatial point;      */
    thee->sym[0][0]    = 0;             /* symmetries of bilinear form      */
    thee->est[0]       = 1.0;           /* error estimator weights          */
    for (i=0; i<VMAX_BDTYPE; i++)       /* boundary type remappings         */
        thee->bmap[0][i] = i;

    /* Manifold-specific function pointers */
    thee->bisectEdge  = bisectEdge;  /* edge bisection rule                 */
    thee->mapBoundary = mapBoundary; /* boundary recovery rule              */
    thee->markSimplex = markSimplex; /* simplex marking rule                */
    thee->oneChart    = oneChart;    /* coordinate transformations          */

    /* apbs stuff */
    thee->user = usr_ctor();

    /* return the new pde object */
    return thee;
}
VPUBLIC void myPDE_dtor(PDE *thee) {

    VASSERT( (thee) != VNULL );
    if ((thee) != VNULL) {
        usr_dtor(thee->user);
        Vmem_free(VNULL, 1, sizeof(PDE), (void **)&thee);
        thee = VNULL;
    }
}

/*
VPUBLIC double PDE_getTime(PDE *thee)
{
    if( thee->user != VNULL ) {
        return ((VlinesPDE *)(thee->user))->time;
    } else {
        return 0.0;
    }
}
*/

VPUBLIC Vusr* usr_ctor(){
    Vusr *thee; 

    thee = Vmem_malloc(VNULL,1,sizeof(Vusr));
    thee->model = model_ctor();
    thee->lparam= lparam_ctor();
    thee->tparam= tparam_ctor();

    return thee;
}

VPUBLIC void usr_dtor(Vusr *thee){
    model_dtor(thee->model);
    lparam_dtor(thee->lparam);
    tparam_dtor(thee->tparam);
    Vmem_free(VNULL, 1, sizeof(Vusr), (void **)(&(thee)));
    thee = VNULL;
} 

VPUBLIC Vmodel* model_ctor(){
    Vmodel *thee;
    thee = Vmem_malloc( VNULL, 1, sizeof(Vmodel) );
    thee->alist = Valist_ctor();
    thee->mgrid = Vmgrid_ctor();
    return thee;
}

VPUBLIC void model_dtor(Vmodel *thee){
    Vmem_free(VNULL, 1, sizeof(Vmodel), (void **)(&(thee)));
}

//VPUBLIC Vmodel* usr_getModel(PDE *thee){return ((Vusr *)thee->user)->model;}
//VPUBLIC Vlparam* usr_getLparam(PDE *thee){return ((Vusr *)thee->user)->lparam;}
//VPUBLIC Vtparam* usr_getTparam(PDE *thee){return ((Vusr *)thee->user)->tparam;}
