/**
 *  @file mcsf2off.c
 *  @ingroup mcsf2off
 *  @author  Nathan Baker
 *  @brief  Converts MCSF-format meshes to Geomview OFF format
 *  @version $Id: mcsf2off.c,v 1.1.1.1 2003/06/24 15:57:11 smol Exp $
 */

#include "smol.h"  

#include "./mcsf2off.h"

VEMBED(rcsid="$Id: mcsf2off.c,v 1.1.1.1 2003/06/24 15:57:11 smol Exp $")

VPRIVATE void usage(int rc) {

    char *ustr = "\n\
  Converts FEtk MCSF-format meshes to Geomview OFF format.\n\
  Usage:\n\
      mcsf2off [options] <mesh.m>\n\
  where <mesh.m> is the input mesh in MCSF format and [options]\n\
  includes:\n\
      --check     Check the validity of the input mesh\n\
      --boundary  Only print the boundaries of the mesh\n\
      --help      Print help message\n\
  The resulting OFF-format mesh is written to <mesh>.off, where\n\
  <mesh> is the stem of the input mesh.\n\n";

    Vnm_print(2, ustr);
    exit(rc);

}

VPRIVATE void header() {

    char *hstr = "\n\n\
 ------------------------------------------------------------------------\n\
 mcsf2off -- A progam to convert MCSF-format meshes to Geomview OFF\n\
 format.\n\n\
 Nathan Baker\n\
 E-mail:  baker@biochem.wustl.edu\n\
 Web:  http://agave.wustl.edu/\n\
 ------------------------------------------------------------------------\n\
 \n";

    Vnm_print(1, hstr);
}

VPRIVATE int getopt(int argc, char **argv, char mcsf_path[VMAX_ARGLEN],
  int *doCheck, int *fullMesh) {

    int i, nstr, isOpt, gotPath;
    char *arg;

    *doCheck = 0;
    *fullMesh = 1;
    gotPath = 0;

    for (i=1; i<argc; i++) {
        /* Get the argument */
        arg = argv[i];
        nstr = strlen(arg);
        /* See if this is an option or an argument */
        isOpt = 0;
        if (nstr > 2) {
            if ((arg[0] == '-') && (arg[1] == '-')) isOpt = 1;
        }
        if (isOpt) {
            if (strcmp(arg, "--check") == 0) {
                *doCheck = 1;
            } else if (strcmp(arg, "--boundary") == 0) {
                *fullMesh = 0;
            } else if (strcmp(arg, "--help") == 0) {
                usage(0);
            } else {
                Vnm_print(2, "invalid option:  %s\n", arg);
                return 0;
            }
        } else {
            if (gotPath == 1) {
                Vnm_print(2, "Already got path (%s); what's this (%s)?\n",
                  mcsf_path, arg);
                return 0;
            } else {
                if (nstr > VMAX_ARGLEN) {
                    Vnm_print(2, "Argument too long: %s\n", arg);
                    return 0;
                }
                strcpy(mcsf_path, arg);
                gotPath = 1;
            }
        }
    }

    if (!gotPath) {
        Vnm_print(2, "No input path specified!\n");
        return 0;
    }

    return 1;
}

int main(int argc, char **argv) {

    Gem *gm = VNULL;
    PDE *pde = VNULL;
    Vio *sock = VNULL;
    char mcsf_path[VMAX_ARGLEN];
    char *off_path;
    int i, rc, nstr, nalloc, doCheck, fullMesh;

    /* Start the I/O layer */
    Vio_start();
    header();

    /* Check invocation */
    rc = getopt(argc, argv, mcsf_path, &doCheck, &fullMesh);
    if (!rc) {
        Vnm_print(2, "\nOption error!\n");
        usage(2);
    }

    /* Transform mcsf_path into off_path; allocate enough space for the string,
     * less the .m extension, plus the \0, plus the new .off extension */
    nstr = strlen(mcsf_path);
    nalloc = nstr - 2 + 1 + 4;
    off_path = (char *)Vmem_malloc(VNULL, nalloc, sizeof(char));
    for (i=0; i<(nstr-2); i++) off_path[i] = mcsf_path[i];
    off_path[nstr-2] = '.';
    off_path[nstr-1] = 'o';
    off_path[nstr-0] = 'f';
    off_path[nstr+1] = 'f';
    off_path[nstr+2] = '\0';

    Vnm_print(1, "\nParameters:\n");
    Vnm_print(1, "MCSF input: %s\n", mcsf_path);
    Vnm_print(1, "OFF output: %s\n", off_path);
    if (doCheck) Vnm_print(1, "Will check validity of mesh file.\n");
    Vnm_print(1, "Will not check validity of mesh file.\n");
    if (fullMesh) Vnm_print(1, "Will write out full mesh.\n");
    else Vnm_print(1, "Will only write out mesh boundaries.\n");
  
    Vnm_print(1, "\nReading MCSF-format data from %s...\n", mcsf_path);
    pde = myPDE_ctor();
    gm = Gem_ctor(VNULL, pde);
    sock = Vio_ctor("FILE", "ASC", "localhost", mcsf_path, "r");
    Gem_read(gm, 0, sock);
    Vio_dtor(&sock);

    if (doCheck) { 
        Vnm_print(1,"\nChecking mesh quality....\n");
        Vnm_redirect(0);
        Gem_formChk(gm, 3); 
        Gem_shapeChk(gm); 
        Vnm_redirect(1);
    }

    Vnm_print(1, "\nWriting OFF-format data to %s...\n", off_path);
    sock = Vio_ctor("FILE", "ASC", "localhost", off_path, "w");
    Gem_writeGV(gm, sock, 0, 2, -1, 1, (!fullMesh), VNULL);
    Vio_dtor(&sock);

    Vnm_print(1,"\nPerforming garbage collection:\n");
    Vnm_print(1,"Destroying PDE object...\n");
    myPDE_dtor(pde);
    Vnm_print(1,"Destroying Gem object...\n");
    Gem_dtor(&gm);
    Vnm_print(1,"Destroying character array...\n");
    Vmem_free(VNULL, nalloc, sizeof(char), (void **)&off_path);

    Vnm_print(1,"\nStopping execution.\n");
    return 0;
}
