/** @file    mypde.h
 *  @brief   Contains PDE definitions for FEtk
 *  @version $Id: mypde.h,v 1.1.1.1 2003/06/24 15:57:11 smol Exp $
 *  @author   Nathan A. Baker
 */

#ifndef _SMOL_MYPDE_H_
#define _SMOL_MYPDE_H_

/* Generic headers */
#include "maloc/maloc.h"
#include "mc/mc.h"

/**
 * @brief Construct the differential equation object
 * @ingroup PDE
 * @author Mike Holst
 * @returns Newly-constructed object */
VEXTERNC PDE* myPDE_ctor();

/**
 * @brief Destroy the differential equation object
 * @ingroup PDE
 * @author Mike Holst
 * @param thee pointer to the memory location for the object */
VEXTERNC void myPDE_dtor(PDE *thee);

/**
 * @brief Do once-per-assembly initialization
 * @ingroup PDE
 * @author Mike Holst
 * @param thee pointer to PDE object
 * @param ip integer parameters for the assembly
 * @param rp real parameters for the assembly
 * @notes This function is called by MC just before a full assembly,
 * and does not have to be particularly fast.  This function is not used in
 * SMOL. */
VEXTERNC void initAssemble(PDE *thee, int ip[], double rp[]);

/**
 * @brief Do one-per-element initialization
 * @author Nathan Baker and Mike Holst
 * @ingroup PDE
 * @param thee  PDE object
 * @param elementType  type of element (various material types)
 * @param chart  chart in which vertex coordinates are provided
 * @param tvx  coordinates of all the vertices of the element
 * @param data extra user-specified data (such as the simplex object)
 * @notes  This function is called by MC just before assembling a single
 * element and needs to be as fast as possible */
VEXTERNC void initElement(PDE *thee, int elementType, int chart, 
    double tvx[][3], void *data);

/**
 * @brief Do once-per-face initialization
 * @author Nathan Baker and Mike Holst
 * @ingroup PDE
 * @param thee PDE object
 * @param faceType  type of this face (interior or various boundary types)
 * @param chart  chart in which normal vector coordinates are provided
 * @param tnvec  coordinates of the outward normal vector to this face
 * @notes  This function is called by MC just before assembling a single
 * element face and needs to be as fast as possible */
VEXTERNC void initFace(PDE *thee, int faceType, int chart, double tnvec[]);

/**
 * @brief Do once-per-point initialization
 * @author Nathan Baker
 * @ingroup PDE
 * @param thee PDE object
 * @param pointType type of point 
 *   \li 0 => interior
 *   \li odd => Dirichlet boundary
 *   \li even => Neumann boundary
 * @param chart chart in which the point coordinates are provided
 * @param txq coordinates of the point
 * @param tU current solution at the point
 * @param tdU current solution gradient at the point
 * @notes This function is called by MC for every quadrature point
 * during an assmebly, and needs to be a fast as possible. */
VEXTERNC void initPoint(PDE *thee, int pointType, int chart, double txq[], 
    double tU[], double tdU[][3]);

/**
 * @brief Evaluate the strong form of the differential operator F(u) at the
 * single point x.
 *
 * The nonlinear strong form is:
 * @verbatim
 *          [   b(u)^i - a(u)^{iq}_{~;q},   if t = 0
 *   F(u) = [   c(u)^i + a(u)^{iq} n_q,     if t = 1
 *          [   0      + a(u)^{iq} n_q,     if t = 2
 * @endverbatim
 * where t denotes the type of point (0=interior, 1=boundary, 2=int-bndry).
 *
 * @ingroup PDE
 * @author Nathan Baker and Mike Holst
 * @param thee the PDE object
 * @param key piece to evaluate
 *        \li 0 => interior point residual
 *        \li 1 => boundary residual 
 *        \li 2 => interior boundary residual (i.e., jump term)
 * @param F set to strong form value
 * @notes The current implementation assumes a constant isotropic diffusion
 * coefficient and piecewise linear basis functions */
VEXTERNC void Fu(PDE *thee, int key, double F[]);

/**
 * @brief Evaluate the integrand F(u)(v) of the functional <F(u),v> (i.e., the
 * nonlinear weak form) at the single point x.  
 *
 * The nonlinear weak form is
 * @verbatim
 *                 /\                 /\
 *      <F(u),v> = \  F_0(u)(v) dx +  \  F_1(u)(v) ds
 *                \/m                \/dm
 * 
 *                 /\
 *               = \  g_{ij} ( a(u)^{iq} v^j_{~;q} + b(u)^i v^j ) dx
 *                \/m
 * 
 *                 /\
 *               + \  g_{ij} c(u)^i v^j ds
 *                \/dm
 * @endverbatim
 * @ingroup PDE
 * @author Nathan Baker and Mike Holst
 * @param thee the PDE object
 * @param key integrand to evaluate 
 *        \li 0 => F_0 (the interior form)
 *        \li 1 => F_1 (the boundary form)
 * @param V test function value at the current point
 * @param dV test function gradient at the current point
 * @returns value of the integrand
 * @notes This function is called by MC multiple times for a single
 * quadrature point during assembly, and needs to be a fast
 * as possible.
 */
VEXTERNC double Fu_v(PDE *thee, int key, double V[], double dV[][3]);

/**
 * @brief  Evaluate the integrand DF(u)(w,v) of the functional <DF(u)w, v>
 * (i.e., the Gateaux derivative or the linearization of the weak form around
 * the current solution) at the single point x.
 *
 * This nonlinear weak form is:
 * @verbatim
 *                   /\                    /\
 *      <DF(u)w,v> = \  DF_0(u)(w,v) dx +  \  DF_1(u)(w,v) ds
 *                  \/m                   \/dm
 * 
 *                   /\
 *                 = \  d/dt F_0(u+tw)(v)|_{t=0} dx
 *                  \/m
 * 
 *                   /\
 *                 + \  d/dt F_1(u+tw)(v)|_{t=0} ds
 *                  \/dm
 * @endverbatim
 * @ingroup PDE
 * @author Nathan Baker and Mike Holst
 * @param thee the PDE object
 * @param  key integrand to evaluate 
 *         \li 0 => DF_0 (the interior form)
 *         \li 1 => DF_1 (the boundary form)
 * @param W trial function at the current point
 * @param dW trial function gradient at the current point
 * @param V test function at the current point
 * @param dV test function gradient at the current point
 * @returns Value of the integrand
 * @notes This function is called by MC multiple times for a single
 * quadrature point during assembly, and needs to be a fast
 * as possible.
 */
VEXTERNC double DFu_wv(PDE *thee, int key, double W[], double dW[][3],
    double V[], double dV[][3]);

/**
 * @brief  At the single point x, evaluate a delta function source term if one
 * is present.
 * @ingroup PDE
 * @author Nathan Baker
 * @param thee pointer to PDE object
 * @param type type of point 
 *   \li 0 => interior
 *   \li odd => Dirichlet boundary
 *   \li even => Neumann boundary
 * @param chart chart in which the point coordinates are provided
 * @param txq coordinates of the point
 * @param F  resulting function value.
 * @param data extra user-specified data such as the vertex object
 * @notes This function is called by MC once for each node in the mesh,
 * just after a full element-wise assembly, so it should be fast.  This
 * function is not used in SMOL.
 */
VEXTERNC void delta(PDE *thee, int type, int chart, double txq[], 
    void *data, double F[]);

/**
 * @brief Evaluate the Dirichlet boundary function at the point x.
 * @ingroup PDE
 * @author Nathan Baker
 * @param thee the PDE object
 * @param type vertex (boundary) type
 *   \li 0 => interior
 *   \li odd => Dirichlet boundary
 *   \li even => Robin/Neumann boundary
 * @param chart chart in which the point coordinates are provided
 * @param txq coordinates of the point
 * @param F set to resulting function values
 * @notes This function is called by MC just before a full assembly,
 * and does not have to be particularly fast.
 */
VEXTERNC void u_D(PDE *thee, int type, int chart, double txq[], double F[]);

/**
 * @brief Evaluate the true solution (if known, for error estimation) function
 * at the point x.  
 * @ingroup PDE
 * @author Nathan Baker
 * @param thee the PDE object
 * @param type vertex (boundary) type
 *    \li 0 => interior
 *    \li odd => Dirichlet boundary
 *    \li even => Robin/Neumann boundary
 * @param chart chart in which the point coordinates are provided
 * @param txq coordinates of the point
 * @param F set to resulting function values
 * @notes This function is called by MC just before a full assembly,
 * and does not have to be particularly fast.
 */
VEXTERNC void u_T(PDE *thee, int type, int chart, double txq[], double F[]);

/**
 * @brief Define the way manifold edges are bisected
 * @ingroup PDE
 * @author Mike Holst
 * @param dim  intrinsic dimension of the manifold 
 * @param dimII  imedding dimension of the manifold 
 * @param edgeType  edge type being refined
 * @param chart manifold charts for coordinates of vertices of edge
 * @param vx coordinates for vertices of edge
 * @notes This function is called by MC every time an edge must be
 * bisected for simplex refinement, and needs to be as fast as
 * possible.
 */
VEXTERNC void bisectEdge(int dim, int dimII, int edgeType, int chart[], 
    double vx[][3]);

/**
 * @brief Map boundaries to a specific shape
 * @ingroup PDE
 * @author Nathan Baker
 * @param dim  intrinsic dimension of the manifold 
 * @param dimII  imedding dimension of the manifold 
 * @param vertexType  vertex type being relocated (i.e., interior or specific
 * boundary type)
 * @param chart manifold charts for coordinates of vertices of edge
 * @param vx coordinates for vertices of edge
 * @notes Right now, if the variable domRad of LocalVars is greater than 0, we
 * map the exterior Dirichlet boundary to a sphere of radius domRad.
 */
VEXTERNC void mapBoundary(int dim, int dimII, int vertexType, int chart, 
    double vx[3]);

/**
 * @brief User-provided error estimator which allows the user to define
 * his own refinement strategies rather than using the builtin a posteriori
 * error estimators.
 * @ingroup PDE
 * @author Nathan Baker
 * @param dim intrinsic dimension of the manifold 
 * @param dimII  imedding dimension of the manifold 
 * @param chart manifold charts for coordinates of vertices of edge
 * @param vx coordinates for vertices of edge
 * @param simplexType type of simplex
 * @param vertexType type of vertex
 * @param faceType type of face
 * @param data a pointer to the simplex object (or VNULL)
 * @returns 1 if the simplex should be refined, 0 otherwise
 * @notes This function is called by MC for every element during error
 * estimation, if the user-provided error estimator is requested.
 * Therefore, it should be pretty fast or the error estimation
 * phase will be slow.
 */
VEXTERNC int markSimplex(int dim, int dimII, int simplexType, 
    int faceType[4], int vertexType[4], int chart[], double vx[][3], 
    void *data);

/**
 * @brief Select a single unified chart for a set of two or more vertices whose
 * coordinates may be given with respect to different charts.  Then transform
 * all of the coordinates of the vertex set to be with respect to the single
 * selected "unified" chart.
 * @ingroup PDE
 * @author Mike Holst
 * @param dim intrinsic dimension of the manifold
 * @param dimII imbedding dimension of the manifold
 * @param dimV number of vertices in the simplex
 *    \li 1 => v is a single vertex
 *    \li 2 => v is part of an edge
 *    \li 3 => v is part of a triangle (or face)
 *    \li 4 => v is part of a tetrahedron
 * @param objType object type of the simplex
 *    \li 0 => object is interior edge/face
 *    \li ODD => object is Dirichlet edge/face
 *    \li EVEN => object is Neumann edge/face
 * @param chart current manifold chart numbers for input vertices
 * @param vx original vertex coordinates w.r.t. charts in chart[]
 * @notes:  This function is not used in SMOL.
 * <p>
 * This function is called by MC whenever it encounters an element with
 * vertices having coordinates in different charts.  If you many charts and
 * thus many elements which stradle chart boundaries, then you will want to
 * make this as fast as possible.
 * <p>
 * In this routine, you define how to move from one chart to another, which
 * means you implicitly define how to change coordinate systems.  The code
 * allows the user to select the appropriate chart for the vertices of an
 * object in order to give the user maximal control over how he uses coordinate
 * systems to represent a manifold.
 * <p>
 * The caller gives the user a set of vertex coordinates for two or more
 * vertices that are a part of some simplex (edge, triangle, or tet).  The
 * vertex coordinates may be given with respect to different charts if the
 * simplex happens to stradle two charts.
 * <p>
 * Our job is to select the most appropriate chart based on the simplex class
 * (edge, tri, or tet) and the simplex type (interior or boundary for edges, or
 * triangular boundary faces, or some material type for triangles or tets), and
 * then transform all of the vertex coordinates to the single coordinate system
 * you have selected.  You then pass back the chart number that you have
 * selected, along with the new coordinates.
 */
VEXTERNC void oneChart(int dim, int dimII, int objType, int chart[], 
    double vx[][3], int dimV);

#endif /* ifndef _SMOL_MYPDE_H_ */
